﻿import mx.controls.Button;
import mx.controls.NumericStepper;
import mx.utils.Delegate;
import flash.filters.ColorMatrixFilter;
import com.gskinner.transitions.MultiTween;
import com.gskinner.geom.ColorMatrix;

class ColorMatrixDemo extends MovieClip {
// private props:
	private var mTween:MultiTween;
	private var cm:ColorMatrix;
	private var endCM:ColorMatrix;
	private var movePos:Number = 0;
	private var resetIntervalID:Number;
// ui elements:
	private var tween1Btn:Button;
	private var tween2Btn:Button;
	private var brightStart:NumericStepper;
	private var contrastStart:NumericStepper;
	private var saturationStart:NumericStepper;
	private var hueStart:NumericStepper;
	private var brightEnd:NumericStepper;
	private var contrastEnd:NumericStepper;
	private var saturationEnd:NumericStepper;
	private var hueEnd:NumericStepper;
	private var flowersStart:MovieClip;
	private var flowersEnd:MovieClip;
	
// initialization:
	function onLoad():Void {
		tween1Btn.addEventListener("click",this);
		tween2Btn.addEventListener("click",this);
		
		brightStart.addEventListener("change",Delegate.create(this,startColorChange));
		contrastStart.addEventListener("change",Delegate.create(this,startColorChange));
		saturationStart.addEventListener("change",Delegate.create(this,startColorChange));
		hueStart.addEventListener("change",Delegate.create(this,startColorChange));
		brightEnd.addEventListener("change",Delegate.create(this,endColorChange));
		contrastEnd.addEventListener("change",Delegate.create(this,endColorChange));
		saturationEnd.addEventListener("change",Delegate.create(this,endColorChange));
		hueEnd.addEventListener("change",Delegate.create(this,endColorChange));
	}

// callbacks:
	// Update previews
	function startColorChange(p_evtObj:Object):Void {
		updateStartCM();
		flowersStart.filters = [new ColorMatrixFilter(cm)];
	}
	
	function endColorChange(p_evtObj:Object):Void {
		updateEndCM();
		flowersEnd.filters = [new ColorMatrixFilter(endCM)];
	}
	
	function click(p_evtObj:Object):Void {
		clearInterval(resetIntervalID);
		delete(resetIntervalID);
		// Set up the ColorMatrix to tween
		updateStartCM();
		// Set up the target ColorMatrix values
		updateEndCM();
		// Set up the MultiTween
		mTween = new MultiTween(cm,endCM);
		// Diff tween types
		if (p_evtObj.target == tween1Btn) {
			onEnterFrame = colorTween;
		} else {
			onEnterFrame = colorTweenSpring;
		}
	}

// private methods:	
	// Deceleratory Tween
	private function colorTween() {
		// Simple Deleceration calculation
		mTween.position += (1 - mTween.position)/10;
		// Kill the onEnterFrame if we've reached the target
		if (Math.round(mTween.position*100) >= 100) { 
			endTween();
		}
		// Apply the filter to the clip
		flowersStart.filters = [new ColorMatrixFilter(cm)];
	}
	
	// "Spring-like" Tween
	private function colorTweenSpring() {
		// Spring motion calculation
		movePos += (1 - mTween.position)/5;
		movePos *= 0.85;
		mTween.position += movePos;
		// Kill the onEnterFrame if we've reached the target
		if (Math.round(mTween.position*100) == 100 && Math.round(movePos*100) == 0) { 
			endTween();
		}
		// Apply the filter to the clip
		flowersStart.filters = [new ColorMatrixFilter(cm)];
	}
	
	// Update ColorMatrices
	private function updateStartCM() {
		cm = new ColorMatrix();
		cm.adjustColor(brightStart.value,contrastStart.value,saturationStart.value,hueStart.value);
	}
	
	private function updateEndCM() {
		endCM = new ColorMatrix();
		endCM.adjustColor(brightEnd.value,contrastEnd.value,saturationEnd.value,hueEnd.value);
	}
	
	// Kill the onEnterFrame and reset the start image
	private function endTween() {
		cm = endCM;
		delete onEnterFrame; 
		resetIntervalID = setInterval(this,"resetStart",1000);
	}
	
	private function resetStart() {
		updateStartCM();
		flowersStart.filters = [new ColorMatrixFilter(cm)];
		clearInterval(resetIntervalID);
		delete(resetIntervalID);
	}
}